﻿namespace iLoan.Rules;

public class RuleFileGenerator(DomainContext context, int startFromId, string root, int seqNo)
{
    public async Task GenerateRuleFilesAsync()
    {
        var rules = await context.CoreRules.Where(rule => rule.Id >= startFromId).ToListAsync();

        foreach (var rule in rules.OrderBy(x =>x.Id))
        {
            var fileName = $"V2.{seqNo:000}__{rule.WorkItemId}_Rules_{rule.Id}_{rule.Title}.sql";
            var sql = GenerateSqlStatement(rule);
            // Simulate writing to a file (you can replace this with actual file writing)
            Console.WriteLine($"Generating file: {fileName}");
            await File.WriteAllTextAsync(Path.Combine(root, fileName), sql, Encoding.UTF8);
            seqNo++;
        }
    }

    private string GenerateSqlStatement(CoreRule rule)
    {
        return $@"
-- SQL statement to insert or update the rule
INSERT INTO core.""Core_Rules"" (
    ""ID"", ""RuleName"", ""Source"", ""DefaultThreshold"", ""Blocked"", ""TriggersManualReview"", 
    ""SuggestedResolution"", ""TriggerReason"", ""UseLastRecord"", ""ApplicantType"", ""FilterOnClickthrough"",
    ""RunBeforeStatus"", ""DisplaySection"", ""WarningField"", ""DutchRuleName"", ""DutchTriggerReason"", 
    ""DutchSuggestedResolution"", ""DutchFilterOnClickthrough"", ""RuleExplanation"", ""DutchRuleExplanation""
) OVERRIDING SYSTEM VALUE VALUES (
    {rule.Id}, '{EscapeSingleQuote(rule.RuleName)}', '{EscapeSingleQuote(rule.Source)}', {rule.DefaultThreshold?.ToString() ?? "NULL"}, 
    {rule.Blocked.ToString().ToUpper()}, {rule.TriggersManualReview.ToString().ToUpper()}, '{EscapeSingleQuote(rule.SuggestedResolution)}', 
    '{EscapeSingleQuote(rule.TriggerReason)}', {rule.UseLastRecord?.ToString().ToUpper() ?? "NULL"}, '{EscapeSingleQuote(rule.ApplicantType)}', 
    '{EscapeSingleQuote(rule.FilterOnClickthrough)}', '{EscapeSingleQuote(rule.RunBeforeStatus)}', '{EscapeSingleQuote(rule.DisplaySection)}', 
    '{EscapeSingleQuote(rule.WarningField)}', '{EscapeSingleQuote(rule.DutchRuleName)}', '{EscapeSingleQuote(rule.DutchTriggerReason)}', 
    '{EscapeSingleQuote(rule.DutchSuggestedResolution)}', '{EscapeSingleQuote(rule.DutchFilterOnClickthrough)}', 
    '{EscapeSingleQuote(rule.RuleExplanation)}', '{EscapeSingleQuote(rule.DutchRuleExplanation)}'
);
";
    }

    private static string? EscapeSingleQuote(string value)
    {
        return value?.Replace("'", "''");
    }
}